package sun.security.x509;

import sun.security.util.ObjectIdentifier;
import sun.security.util.DerOutputStream;
import java.util.Iterator;
import java.io.IOException;
import sun.security.util.DerValue;
import java.util.ArrayList;
import java.util.List;

public class GeneralSubtrees implements Cloneable
{
    private final List<GeneralSubtree> trees;
    private static final int NAME_DIFF_TYPE = -1;
    private static final int NAME_MATCH = 0;
    private static final int NAME_NARROWS = 1;
    private static final int NAME_WIDENS = 2;
    private static final int NAME_SAME_TYPE = 3;
    
    public GeneralSubtrees() {
        this.trees = new ArrayList();
    }
    
    private GeneralSubtrees(GeneralSubtrees generalSubtrees) {
        this.trees = new ArrayList(generalSubtrees.trees);
    }
    
    public GeneralSubtrees(DerValue derValue) throws IOException {
        this();
        if (derValue.tag != 48) {
            throw new IOException("Invalid encoding of GeneralSubtrees.");
        }
        while (derValue.data.available() != 0) {
            this.add(new GeneralSubtree(derValue.data.getDerValue()));
        }
        return;
    }
    
    public GeneralSubtree get(int n) {
        return (GeneralSubtree)this.trees.get(n);
    }
    
    public void remove(int n) {
        this.trees.remove(n);
    }
    
    public void add(GeneralSubtree generalSubtree) {
        if (generalSubtree == null) {
            throw new NullPointerException();
        }
        this.trees.add(generalSubtree);
        return;
    }
    
    public boolean contains(GeneralSubtree generalSubtree) {
        if (generalSubtree == null) {
            throw new NullPointerException();
        }
        return this.trees.contains(generalSubtree);
    }
    
    public int size() {
        return this.trees.size();
    }
    
    public Iterator<GeneralSubtree> iterator() {
        return this.trees.iterator();
    }
    
    public List<GeneralSubtree> trees() {
        return this.trees;
    }
    
    public Object clone() {
        return new GeneralSubtrees(this);
    }
    
    @Override
    public String toString() {
        return "   GeneralSubtrees:\n" + this.trees.toString() + "\n";
    }
    
    public void encode(DerOutputStream derOutputStream) throws IOException {
        DerOutputStream derOutputStream2 = new DerOutputStream();
        for (int i = 0; i < this.size(); ++i) {
            this.get(i).encode(derOutputStream2);
        }
        derOutputStream.write((byte)48, derOutputStream2);
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof GeneralSubtrees && this.trees.equals(((GeneralSubtrees)o).trees));
    }
    
    @Override
    public int hashCode() {
        return this.trees.hashCode();
    }
    
    private GeneralNameInterface getGeneralNameInterface(int n) {
        return getGeneralNameInterface(this.get(n));
    }
    
    private static GeneralNameInterface getGeneralNameInterface(GeneralSubtree generalSubtree) {
        return generalSubtree.getName().getName();
    }
    
    private void minimize() {
        for (int i = 0; i < this.size() - 1; ++i) {
            GeneralNameInterface generalNameInterface = this.getGeneralNameInterface(i);
            boolean n = false;
            int j = i + 1;
            while (j < this.size()) {
                Label_0112: {
                    switch (generalNameInterface.constrains(this.getGeneralNameInterface(j))) {
                        case -1: {
                            break;
                        }
                        case 0: {
                            n = true;
                            break Label_0112;
                        }
                        case 1: {
                            this.remove(j);
                            --j;
                            break;
                        }
                        case 2: {
                            n = true;
                            break Label_0112;
                        }
                        case 3: {
                            break;
                        }
                    }
                    ++j;
                    continue;
                }
                break;
            }
            if (!(n == 0)) {
                this.remove(i);
                --i;
            }
        }
    }
    
    private GeneralSubtree createWidestSubtree(GeneralNameInterface generalNameInterface) {
        try {
            GeneralName generalName;
            switch (generalNameInterface.getType()) {
                case 0: {
                    generalName = new GeneralName(new OtherName(((OtherName)generalNameInterface).getOID(), null));
                    break;
                }
                case 1: {
                    generalName = new GeneralName(new RFC822Name(""));
                    break;
                }
                case 2: {
                    generalName = new GeneralName(new DNSName(""));
                    break;
                }
                case 3: {
                    generalName = new GeneralName(new X400Address((byte[])null));
                    break;
                }
                case 4: {
                    generalName = new GeneralName(new X500Name(""));
                    break;
                }
                case 5: {
                    generalName = new GeneralName(new EDIPartyName(""));
                    break;
                }
                case 6: {
                    generalName = new GeneralName(new URIName(""));
                    break;
                }
                case 7: {
                    generalName = new GeneralName(new IPAddressName((byte[])null));
                    break;
                }
                case 8: {
                    generalName = new GeneralName(new OIDName(new ObjectIdentifier((int[])null)));
                    break;
                }
                default: {
                    throw new IOException("Unsupported GeneralNameInterface type: " + generalNameInterface.getType());
                }
            }
            return new GeneralSubtree(generalName, 0, -1);
        }
        catch (IOException ex) {
            throw new RuntimeException("Unexpected error: " + ex, ex);
        }
    }
    
    public GeneralSubtrees intersect(GeneralSubtrees generalSubtrees) {
        if (generalSubtrees == null) {
            throw new NullPointerException("other GeneralSubtrees must not be null");
        }
        GeneralSubtrees generalSubtrees2 = new GeneralSubtrees();
        Object o = null;
        if (this.size() == 0) {
            this.union(generalSubtrees);
            return null;
        }
        this.minimize();
        generalSubtrees.minimize();
        for (int i = 0; i < this.size(); ++i) {
            GeneralNameInterface generalNameInterface = this.getGeneralNameInterface(i);
            boolean n = false;
            int j = 0;
        Label_0186:
            while (j < generalSubtrees.size()) {
                GeneralSubtree value = generalSubtrees.get(j);
                switch (generalNameInterface.constrains(getGeneralNameInterface(value))) {
                    case 1: {
                        this.remove(i);
                        --i;
                        generalSubtrees2.add(value);
                        n = false;
                        break Label_0186;
                    }
                    case 3: {
                        n = true;
                        ++j;
                        continue;
                    }
                    case 0:
                    case 2: {
                        n = false;
                        break Label_0186;
                    }
                    default: {}
                }
            }
            if (!(n == 0)) {
                boolean n2 = false;
                for (int k = 0; k < this.size(); ++k) {
                    GeneralNameInterface generalNameInterface2 = this.getGeneralNameInterface(k);
                    if (generalNameInterface2.getType() == generalNameInterface.getType()) {
                        int l = 0;
                        while (l < generalSubtrees.size()) {
                            int constrains = generalNameInterface2.constrains(generalSubtrees.getGeneralNameInterface(l));
                            if (constrains == 0 || constrains == 2 || constrains == 1) {
                                n2 = true;
                                break;
                            }
                            ++l;
                        }
                    }
                }
                if (n2 == 0) {
                    if (o == null) {
                        o = new GeneralSubtrees();
                    }
                    GeneralSubtree widestSubtree = this.createWidestSubtree(generalNameInterface);
                    if (!o.contains(widestSubtree)) {
                        o.add(widestSubtree);
                    }
                }
                this.remove(i);
                --i;
            }
        }
        if (generalSubtrees2.size() > 0) {
            this.union(generalSubtrees2);
        }
        for (int n3 = 0; n3 < generalSubtrees.size(); ++n3) {
            GeneralSubtree value2 = generalSubtrees.get(n3);
            GeneralNameInterface generalNameInterface3 = getGeneralNameInterface(value2);
            boolean n4 = false;
            int n5 = 0;
        Label_0477:
            while (n5 < this.size()) {
                switch (this.getGeneralNameInterface(n5).constrains(generalNameInterface3)) {
                    case -1: {
                        n4 = true;
                        ++n5;
                        continue;
                    }
                    case 0:
                    case 1:
                    case 2:
                    case 3: {
                        n4 = false;
                        break Label_0477;
                    }
                    default: {}
                }
            }
            if (!(n4 == 0)) {
                this.add(value2);
            }
        }
        return o;
    }
    
    public void union(GeneralSubtrees generalSubtrees) {
        if (generalSubtrees != null) {
            for (int i = 0; i < generalSubtrees.size(); ++i) {
                this.add(generalSubtrees.get(i));
            }
            this.minimize();
        }
    }
    
    public void reduce(GeneralSubtrees generalSubtrees) {
        if (generalSubtrees == null) {
            return;
        }
        for (int i = 0; i < generalSubtrees.size(); ++i) {
            GeneralNameInterface generalNameInterface = generalSubtrees.getGeneralNameInterface(i);
            for (int j = 0; j < this.size(); ++j) {
                switch (generalNameInterface.constrains(this.getGeneralNameInterface(j))) {
                    case -1: {
                        break;
                    }
                    case 0: {
                        this.remove(j);
                        --j;
                        break;
                    }
                    case 1: {
                        this.remove(j);
                        --j;
                        break;
                    }
                    case 2: {}
                }
            }
        }
        return;
    }
}
